require 'spec_helper'

describe Endorser::Client do
  let(:client) { Endorser::Client.new('abc', '123', 'My Realm') }

  describe '#initialize' do
    subject { client }
    its(:key) { should eq('abc') }
    its(:secret) { should eq('123') }
  end

  describe '#get' do
    it 'should execute a GET request' do
      url = 'http://localhost:3000/'
      headers = { data: 'headers' }
      client.stub(execute: true)

      client.get(url, headers)

      expect(client).to have_received(:execute).with({
        method: :get,
        url: url,
        headers: headers
      })
    end
  end

  describe '#put' do
    it 'should execute a PUT request' do
      url = 'http://localhost:3000/'
      payload = { data: 'payload'}
      headers = { data: 'headers' }
      client.stub(execute: true)

      client.put(url, payload, headers)

      expect(client).to have_received(:execute).with({
        method: :put,
        url: url,
        headers: headers,
        payload: payload
      })
    end
  end

  describe '#post' do
    it 'should execute a POST request' do
      url = 'http://localhost:3000/'
      payload = { data: 'payload'}
      headers = { data: 'headers' }
      client.stub(execute: true)

      client.post(url, payload, headers)

      expect(client).to have_received(:execute).with({
        method: :post,
        url: url,
        headers: headers,
        payload: payload
      })
    end
  end

  describe '#delete' do
    it 'should execute a DELETE request' do
      url = 'http://localhost:3000/'
      headers = { data: 'headers' }
      client.stub(execute: true)

      client.delete(url, headers)

      expect(client).to have_received(:execute).with({
        method: :delete,
        url: url,
        headers: headers
      })
    end
  end

  describe '#signed_request' do
    context 'when it is a GET request' do
      it 'should sign the request' do
        args = {
          method: :get,
          url: 'http://localhost:3000'
        }
        response = client.signed_request(args)
      end
    end
  end

  describe '#uri_path_and_query' do
    context 'when the url is "http://example.com:3000/"' do
      let(:url) { "http://localhost:3000/" }
      subject { client.uri_path_and_query(url) }
      it { should eq("/")}
    end

    context 'when the url is "http://example.com/something/different"' do
      let(:url) { "http://example.com/something/different" }
      subject { client.uri_path_and_query(url) }
      it { should eq("/something/different")}
    end

    context 'when the url is "http://example.com:3000/?q=abc"' do
      let(:url) { "http://localhost:3000/?q=abc" }
      subject { client.uri_path_and_query(url) }
      it { should eq("/?q=abc")}
    end

    context 'when the url is "http://example.com/something/different?q=abc"' do
      let(:url) { "http://example.com/something/different?q=abc" }
      subject { client.uri_path_and_query(url) }
      it { should eq("/something/different?q=abc")}
    end
  end

  describe '#host' do
    context 'when the url is "http://example.com:3000/"' do
      let(:url) { "http://localhost:3000/" }
      subject { client.host(url) }
      it { should eq("http://localhost:3000")}
    end

    context 'when the url is "https://example.com/"' do
      let(:url) { "https://example.com/" }
      subject { client.host(url) }
      it { should eq("https://example.com")}
    end

    context 'when the url is "http://example.com/something/different"' do
      let(:url) { "http://example.com/something/different" }
      subject { client.host(url) }
      it { should eq("http://example.com")}
    end

    context 'when the url is "http://example.com:3000/something/different"' do
      let(:url) { "http://example.com:3000/something/different" }
      subject { client.host(url) }
      it { should eq("http://example.com:3000")}
    end
  end

  describe '#include_port?' do
    context 'when the scheme is http' do
      context 'and when the port is 80' do
        let(:uri) { double('URI', scheme: 'http', port: 80)}
        subject { client.include_port?(uri) }
        it { should be_false }
      end

      context 'and when the port is 443' do
        let(:uri) { double('URI', scheme: 'http', port: 443)}
        subject { client.include_port?(uri) }
        it { should be_true }
      end
    end

    context 'when the scheme is https' do
      context 'and when the port is 80' do
        let(:uri) { double('URI', scheme: 'https', port: 80)}
        subject { client.include_port?(uri) }
        it { should be_true }
      end

      context 'and when the port is 443' do
        let(:uri) { double('URI', scheme: 'https', port: 443)}
        subject { client.include_port?(uri) }
        it { should be_false }
      end
    end

    context 'when the scheme is something' do
      context 'and when the port is 80' do
        let(:uri) { double('URI', scheme: 'something', port: 80)}
        subject { client.include_port?(uri) }
        it { should be_true }
      end
    end
  end

  describe '#build_receipt' do
    def args
      {
        method: :post,
        headers: {},
        url: 'http://example.com',
        payload: { data: 'here', another: 'set' }
      }
    end
    subject { client.build_receipt(args) }

    it { should be_a(Endorser::Receipt) }
    its(:host) { should eq("http://example.com") }
    its(:request_method) { should eq('POST') }
    its(:body) { should eq('data=here&another=set') }
    its(:body_md5) { should eq('f4a4a0348d64714e94a3360af1dc614c') }
  end

end
